#include "RootBeerUtil.h"
#include "TThread.h"
#include "TLorentzVector.h"
#include "TVector3.h"

#include <vector>
#include <iostream>
#include <fstream>
#include <string>
#include <cmath>

using namespace std;
#define PI 3.14159

bool ReadInMomCorrs_pro(double [5][13][6]);
bool ReadInMomCorrs_pip(double [12][13][6]);
bool ReadInMomCorrs_pim(double [12][13][6]);
bool ReadInMomCorrs_photon(double [5][13][6]);

TLorentzVector mom_cor_pro(TLorentzVector pro_4vecs_data, double pro_mass) {
  static double proMomCorrs[5][13][6];
  ReadInMomCorrs_pro(proMomCorrs);

  TVector3 temp_pro_3vecs_mom = TVector3(pro_4vecs_data.Px(), pro_4vecs_data.Py(), pro_4vecs_data.Pz());
  double temp_pro_mom_mag = temp_pro_3vecs_mom.Mag();
  Double_t temp_pro_theta = pro_4vecs_data.Theta()*(180/PI);
  Double_t temp_pro_phi = pro_4vecs_data.Phi()*(180/PI);
  int pro_ThetaBin = 0;
  int pro_PhiBin = 0;
  int pro_MomBin = 0;

  if (temp_pro_theta >= 0 && temp_pro_theta < 20) {pro_ThetaBin = 1;}
  else if (temp_pro_theta >= 20 && temp_pro_theta < 30) {pro_ThetaBin = 2;}
  else if (temp_pro_theta >= 30 && temp_pro_theta < 40) {pro_ThetaBin = 3;}
  else if(temp_pro_theta >= 40) {pro_ThetaBin = 4;}

  if (temp_pro_phi < 0) { temp_pro_phi = 360 + temp_pro_phi;}

  if (temp_pro_phi >= 0 && temp_pro_phi < 30) {pro_PhiBin = 1;}
  else if (temp_pro_phi >= 30 && temp_pro_phi < 60) {pro_PhiBin = 2;}
  else if (temp_pro_phi >= 60 && temp_pro_phi < 90) {pro_PhiBin = 3;}
  else if (temp_pro_phi >= 90 && temp_pro_phi < 120) {pro_PhiBin = 4;}
  else if (temp_pro_phi >= 120 && temp_pro_phi < 150) {pro_PhiBin = 5;}
  else if (temp_pro_phi >= 150 && temp_pro_phi < 180) {pro_PhiBin = 6;}
  else if (temp_pro_phi >= 180 && temp_pro_phi < 210) {pro_PhiBin = 7;}
  else if (temp_pro_phi >= 210 && temp_pro_phi < 240) {pro_PhiBin = 8;}
  else if (temp_pro_phi >= 240 && temp_pro_phi < 270) {pro_PhiBin = 9;}
  else if (temp_pro_phi >= 270 && temp_pro_phi < 300) {pro_PhiBin = 10;}
  else if (temp_pro_phi >= 300 && temp_pro_phi < 330) {pro_PhiBin = 11;}
  else if (temp_pro_phi >= 330 && temp_pro_phi < 360) {pro_PhiBin = 12;}

  if ( temp_pro_mom_mag <= 0.5) {pro_MomBin = 1;}
  else if (temp_pro_mom_mag > 0.5 && temp_pro_mom_mag <= 0.6 ) {pro_MomBin = 2;}
  else if (temp_pro_mom_mag > 0.6 && temp_pro_mom_mag <= 0.7 ) {pro_MomBin = 3;}
  else if (temp_pro_mom_mag > 0.7 && temp_pro_mom_mag <= 0.9 ) {pro_MomBin = 4;}
  else if (temp_pro_mom_mag > 0.9) {pro_MomBin = 5;}
  
  //cerr <<" pro_ThetaBin ["<<pro_ThetaBin<<" , "<<temp_pro_theta<<"] pro_PhiBin ["<<pro_PhiBin<<" , "<<temp_pro_phi<<"] pro_MomBin ["<<pro_MomBin<<" , "<<temp_pro_mom_mag<< "]"<<endl; 

  TLorentzVector corr_Pro;
  double pro_mom_Px;
  double pro_mom_Py;
  double pro_mom_Pz;
  double pro_E;

  //pro_mom_Px = pro_4vecs_data.Px()*proMomCorrs[pro_ThetaBin][pro_PhiBin][pro_MomBin];
  //pro_mom_Py = pro_4vecs_data.Py()*proMomCorrs[pro_ThetaBin][pro_PhiBin][pro_MomBin];
  //pro_mom_Pz = pro_4vecs_data.Pz()*proMomCorrs[pro_ThetaBin][pro_PhiBin][pro_MomBin];

  //cerr <<"proMomCorrs[0][0]["<<pro_MomBin<<"] : "<<proMomCorrs[0][0][pro_MomBin]<<endl;

  pro_mom_Px = pro_4vecs_data.Px()*proMomCorrs[0][0][pro_MomBin];
  pro_mom_Py = pro_4vecs_data.Py()*proMomCorrs[0][0][pro_MomBin];
  pro_mom_Pz = pro_4vecs_data.Pz()*proMomCorrs[0][0][pro_MomBin];
  
  pro_E = sqrt(pro_mom_Px*pro_mom_Px + pro_mom_Py*pro_mom_Py + pro_mom_Pz*pro_mom_Pz + pro_mass*pro_mass);

  //cerr << "  compare pro_4vecs_data.E()["<<pro_4vecs_data.E()<<"] ----> pro_E["<<pro_E<<"]"<<endl;

  corr_Pro.SetPx(pro_mom_Px);
  corr_Pro.SetPy(pro_mom_Py);
  corr_Pro.SetPz(pro_mom_Pz);
  corr_Pro.SetE(pro_E);

  return corr_Pro;

}

TLorentzVector mom_cor_pip(TLorentzVector pip_4vecs_data, double pip_mass) {
  static double pipMomCorrs[12][13][6];
  ReadInMomCorrs_pip(pipMomCorrs);

  TVector3 temp_pip_3vecs_mom = TVector3(pip_4vecs_data.Px(), pip_4vecs_data.Py(), pip_4vecs_data.Pz());
  double temp_pip_mom_mag = temp_pip_3vecs_mom.Mag();
  Double_t temp_pip_theta = pip_4vecs_data.Theta()*(180/PI);
  Double_t temp_pip_phi = pip_4vecs_data.Phi()*(180/PI);
  int pip_ThetaBin = 0;
  int pip_PhiBin = 0;
  int pip_MomBin = 0;

  if (temp_pip_theta >= 0 && temp_pip_theta < 20) {pip_ThetaBin = 1;}
  else if (temp_pip_theta >= 20 && temp_pip_theta < 30) {pip_ThetaBin = 2;}
  else if (temp_pip_theta >= 30 && temp_pip_theta < 40) {pip_ThetaBin = 3;}
  else if (temp_pip_theta >= 40 && temp_pip_theta < 50) {pip_ThetaBin = 4;}
  else if (temp_pip_theta >= 50 && temp_pip_theta < 60) {pip_ThetaBin = 5;}
  else if (temp_pip_theta >= 60 && temp_pip_theta < 70) {pip_ThetaBin = 6;}
  else if (temp_pip_theta >= 70 && temp_pip_theta < 80) {pip_ThetaBin = 7;}
  else if (temp_pip_theta >= 80 && temp_pip_theta < 90) {pip_ThetaBin = 8;}
  else if (temp_pip_theta >= 90 && temp_pip_theta < 100) {pip_ThetaBin = 9;}
  else if (temp_pip_theta >= 100 && temp_pip_theta < 110) {pip_ThetaBin = 10;}
  else if (temp_pip_theta >= 110) {pip_ThetaBin = 11;}

  if (temp_pip_phi < 0) { temp_pip_phi = 360 + temp_pip_phi;}

  if (temp_pip_phi >= 0 && temp_pip_phi < 30) {pip_PhiBin = 1;}
  else if (temp_pip_phi >= 30 && temp_pip_phi < 60) {pip_PhiBin = 2;}
  else if (temp_pip_phi >= 60 && temp_pip_phi < 90) {pip_PhiBin = 3;}
  else if (temp_pip_phi >= 90 && temp_pip_phi < 120) {pip_PhiBin = 4;}
  else if (temp_pip_phi >= 120 && temp_pip_phi < 150) {pip_PhiBin = 5;}
  else if (temp_pip_phi >= 150 && temp_pip_phi < 180) {pip_PhiBin = 6;}
  else if (temp_pip_phi >= 180 && temp_pip_phi < 210) {pip_PhiBin = 7;}
  else if (temp_pip_phi >= 210 && temp_pip_phi < 240) {pip_PhiBin = 8;}
  else if (temp_pip_phi >= 240 && temp_pip_phi < 270) {pip_PhiBin = 9;}
  else if (temp_pip_phi >= 270 && temp_pip_phi < 300) {pip_PhiBin = 10;}
  else if (temp_pip_phi >= 300 && temp_pip_phi < 330) {pip_PhiBin = 11;}
  else if (temp_pip_phi >= 330 && temp_pip_phi < 360) {pip_PhiBin = 12;}

  if ( temp_pip_mom_mag <= 0.2) {pip_MomBin = 1;}
  else if (temp_pip_mom_mag > 0.2 && temp_pip_mom_mag <= 0.27 ) {pip_MomBin = 2;}
  else if (temp_pip_mom_mag > 0.27 && temp_pip_mom_mag <= 0.35 ) {pip_MomBin = 3;}
  else if (temp_pip_mom_mag > 0.35 && temp_pip_mom_mag <= 0.53 ) {pip_MomBin = 4;}
  else if (temp_pip_mom_mag > 0.53) {pip_MomBin = 5;}

  //cerr <<" pip_ThetaBin ["<<pip_ThetaBin<<" , "<<temp_pip_theta<<"] pip_PhiBin ["<<pip_PhiBin<<" , "<<temp_pip_phi<<"] pip_MomBin ["<<pip_MomBin<<" , "<<temp_pip_mom_mag<<"]"<<endl;

  TLorentzVector corr_Pip;
  double pip_mom_Px;
  double pip_mom_Py;
  double pip_mom_Pz;
  double pip_E;

  //pip_mom_Px = pip_4vecs_data.Px()*pipMomCorrs[pip_ThetaBin][pip_PhiBin][pip_MomBin];   
  //pip_mom_Py = pip_4vecs_data.Py()*pipMomCorrs[pip_ThetaBin][pip_PhiBin][pip_MomBin];                  
  //pip_mom_Pz = pip_4vecs_data.Pz()*pipMomCorrs[pip_ThetaBin][pip_PhiBin][pip_MomBin];             

  pip_mom_Px = pip_4vecs_data.Px()*pipMomCorrs[0][0][pip_MomBin];
  pip_mom_Py = pip_4vecs_data.Py()*pipMomCorrs[0][0][pip_MomBin];
  pip_mom_Pz = pip_4vecs_data.Pz()*pipMomCorrs[0][0][pip_MomBin];

  pip_E = sqrt(pip_mom_Px*pip_mom_Px + pip_mom_Py*pip_mom_Py + pip_mom_Pz*pip_mom_Pz + pip_mass*pip_mass);

  //cerr << "  compare pip_4vecs_data.E()["<<pip_4vecs_data.E()<<"] ----> pip_E["<<pip_E<<"]"<<endl;

  corr_Pip.SetPx(pip_mom_Px);
  corr_Pip.SetPy(pip_mom_Py);
  corr_Pip.SetPz(pip_mom_Pz);
  corr_Pip.SetE(pip_E);

  return corr_Pip;

}

TLorentzVector mom_cor_pim(TLorentzVector pim_4vecs_data, double pim_mass) {
  static double pimMomCorrs[12][13][6];
  ReadInMomCorrs_pim(pimMomCorrs);

  TVector3 temp_pim_3vecs_mom = TVector3(pim_4vecs_data.Px(), pim_4vecs_data.Py(), pim_4vecs_data.Pz());
  double temp_pim_mom_mag = temp_pim_3vecs_mom.Mag();
  Double_t temp_pim_theta = pim_4vecs_data.Theta()*(180/PI);
  Double_t temp_pim_phi = pim_4vecs_data.Phi()*(180/PI);
  int pim_ThetaBin = 0;
  int pim_PhiBin = 0;
  int pim_MomBin = 0;

  if (temp_pim_theta >= 0 && temp_pim_theta < 20) {pim_ThetaBin = 1;}
  else if (temp_pim_theta >= 20 && temp_pim_theta < 30) {pim_ThetaBin = 2;}
  else if (temp_pim_theta >= 30 && temp_pim_theta < 40) {pim_ThetaBin = 3;}
  else if (temp_pim_theta >= 40 && temp_pim_theta < 50) {pim_ThetaBin = 4;}
  else if (temp_pim_theta >= 50 && temp_pim_theta < 60) {pim_ThetaBin = 5;}
  else if (temp_pim_theta >= 60 && temp_pim_theta < 70) {pim_ThetaBin = 6;}
  else if (temp_pim_theta >= 70 && temp_pim_theta < 80) {pim_ThetaBin = 7;}
  else if (temp_pim_theta >= 80 && temp_pim_theta < 90) {pim_ThetaBin = 8;}
  else if (temp_pim_theta >= 90 && temp_pim_theta < 100) {pim_ThetaBin = 9;}
  else if (temp_pim_theta >= 100 && temp_pim_theta < 110) {pim_ThetaBin = 10;}
  else if (temp_pim_theta >= 110) {pim_ThetaBin = 11;}

  if (temp_pim_phi < 0) { temp_pim_phi = 360 + temp_pim_phi;}

  if (temp_pim_phi >= 0 && temp_pim_phi < 30) {pim_PhiBin = 1;}
  else if (temp_pim_phi >= 30 && temp_pim_phi < 60) {pim_PhiBin = 2;}
  else if (temp_pim_phi >= 60 && temp_pim_phi < 90) {pim_PhiBin = 3;}
  else if (temp_pim_phi >= 90 && temp_pim_phi < 120) {pim_PhiBin = 4;}
  else if (temp_pim_phi >= 120 && temp_pim_phi < 150) {pim_PhiBin = 5;}
  else if (temp_pim_phi >= 150 && temp_pim_phi < 180) {pim_PhiBin = 6;}
  else if (temp_pim_phi >= 180 && temp_pim_phi < 210) {pim_PhiBin = 7;}
  else if (temp_pim_phi >= 210 && temp_pim_phi < 240) {pim_PhiBin = 8;}
  else if (temp_pim_phi >= 240 && temp_pim_phi < 270) {pim_PhiBin = 9;}
  else if (temp_pim_phi >= 270 && temp_pim_phi < 300) {pim_PhiBin = 10;}
  else if (temp_pim_phi >= 300 && temp_pim_phi < 330) {pim_PhiBin = 11;}
  else if (temp_pim_phi >= 330 && temp_pim_phi < 360) {pim_PhiBin = 12;}

  if (temp_pim_mom_mag <= 0.2) {pim_MomBin = 1;}
  else if (temp_pim_mom_mag > 0.2 && temp_pim_mom_mag <= 0.27 ) {pim_MomBin = 2;}
  else if (temp_pim_mom_mag > 0.27 && temp_pim_mom_mag <= 0.35 ) {pim_MomBin = 3;}
  else if (temp_pim_mom_mag > 0.35 && temp_pim_mom_mag <= 0.53 ) {pim_MomBin = 4;}
  else if (temp_pim_mom_mag > 0.53) {pim_MomBin = 5;}

  //cerr <<" pim_ThetaBin ["<<pim_ThetaBin<<" , "<<temp_pim_theta<<"] pim_PhiBin ["<<pim_PhiBin<<" , "<<temp_pim_phi<<"] pim_MomBin ["<<pim_MomBin<<" , "<<temp_pim_mom_mag<<"]"<<endl;

  TLorentzVector corr_Pim;
  double pim_mom_Px;
  double pim_mom_Py;
  double pim_mom_Pz;
  double pim_E;

  //pim_mom_Px = pim_4vecs_data.Px()*pimMomCorrs[pim_ThetaBin][pim_PhiBin][pim_MomBin];    
  //pim_mom_Py = pim_4vecs_data.Py()*pimMomCorrs[pim_ThetaBin][pim_PhiBin][pim_MomBin];      
  //pim_mom_Pz = pim_4vecs_data.Pz()*pimMomCorrs[pim_ThetaBin][pim_PhiBin][pim_MomBin];          

  pim_mom_Px = pim_4vecs_data.Px()*pimMomCorrs[0][0][pim_MomBin];
  pim_mom_Py = pim_4vecs_data.Py()*pimMomCorrs[0][0][pim_MomBin];
  pim_mom_Pz = pim_4vecs_data.Pz()*pimMomCorrs[0][0][pim_MomBin];

  pim_E = sqrt(pim_mom_Px*pim_mom_Px + pim_mom_Py*pim_mom_Py + pim_mom_Pz*pim_mom_Pz + pim_mass*pim_mass);

  //cerr << "  compare pim_4vecs_data.E()["<<pim_4vecs_data.E()<<"] ----> pim_E["<<pim_E<<"]"<<endl;

  corr_Pim.SetPx(pim_mom_Px);
  corr_Pim.SetPy(pim_mom_Py);
  corr_Pim.SetPz(pim_mom_Pz);
  corr_Pim.SetE(pim_E);

  return corr_Pim;

}


double mom_cor_photon(TLorentzVector pro_4vecs_data, double old_pho_energy) {
  static double photonMomCorrs[5][13][6];
  ReadInMomCorrs_photon(photonMomCorrs);

  TVector3 temp_pro_3vecs_mom = TVector3(pro_4vecs_data.Px(), pro_4vecs_data.Py(), pro_4vecs_data.Pz());
  double temp_pro_mom_mag = temp_pro_3vecs_mom.Mag();
  Double_t temp_pro_theta = pro_4vecs_data.Theta()*(180/PI);
  Double_t temp_pro_phi = pro_4vecs_data.Phi()*(180/PI);
  int pro_ThetaBin = 0;
  int pro_PhiBin = 0;
  int pro_MomBin = 0;

  if (temp_pro_theta >= 0 && temp_pro_theta < 20) {pro_ThetaBin = 1;}
  else if (temp_pro_theta >= 20 && temp_pro_theta < 30) {pro_ThetaBin = 2;}
  else if (temp_pro_theta >= 30 && temp_pro_theta < 40) {pro_ThetaBin = 3;}
  else if(temp_pro_theta >= 40) {pro_ThetaBin = 4;}

  if (temp_pro_phi < 0) { temp_pro_phi = 360 + temp_pro_phi;}

  if (temp_pro_phi >= 0 && temp_pro_phi < 30) {pro_PhiBin = 1;}
  else if (temp_pro_phi >= 30 && temp_pro_phi < 60) {pro_PhiBin = 2;}
  else if (temp_pro_phi >= 60 && temp_pro_phi < 90) {pro_PhiBin = 3;}
  else if (temp_pro_phi >= 90 && temp_pro_phi < 120) {pro_PhiBin = 4;}
  else if (temp_pro_phi >= 120 && temp_pro_phi < 150) {pro_PhiBin = 5;}
  else if (temp_pro_phi >= 150 && temp_pro_phi < 180) {pro_PhiBin = 6;}
  else if (temp_pro_phi >= 180 && temp_pro_phi < 210) {pro_PhiBin = 7;}
  else if (temp_pro_phi >= 210 && temp_pro_phi < 240) {pro_PhiBin = 8;}
  else if (temp_pro_phi >= 240 && temp_pro_phi < 270) {pro_PhiBin = 9;}
  else if (temp_pro_phi >= 270 && temp_pro_phi < 300) {pro_PhiBin = 10;}
  else if (temp_pro_phi >= 300 && temp_pro_phi < 330) {pro_PhiBin = 11;}
  else if (temp_pro_phi >= 330 && temp_pro_phi < 360) {pro_PhiBin = 12;}

  if ( temp_pro_mom_mag <= 0.5) {pro_MomBin = 1;}
  else if (temp_pro_mom_mag > 0.5 && temp_pro_mom_mag <= 0.6 ) {pro_MomBin = 2;}
  else if (temp_pro_mom_mag > 0.6 && temp_pro_mom_mag <= 0.7 ) {pro_MomBin = 3;}
  else if (temp_pro_mom_mag > 0.7 && temp_pro_mom_mag <= 0.9 ) {pro_MomBin = 4;}
  else if (temp_pro_mom_mag > 0.9) {pro_MomBin = 5;}

  double new_pho_energy=0;
  
  // new_pho_energy = old_pho_energy*photonMomCorrs[pro_ThetaBin][pro_PhiBin][pro_MomBin];
  new_pho_energy = old_pho_energy*photonMomCorrs[0][0][pro_MomBin];
  
  return new_pho_energy;
}


bool ReadInMomCorrs_pro(double proMomCorrs[5][13][6]) {
  std::ifstream *inFile = 0;
  int thetaBin,phiBin,momBin;
  double corr_factor = 0.0;
  string fileName = "/d/home/skpark/ANALYSIS/MakNtuple/MomCor/mom_corr_array.pro";
  inFile = new std::ifstream(fileName.c_str());

  if(!(inFile->is_open())){
    std::cout << "Error! <ReadInMomCorrs_pro> File read error: " << fileName << " Unable to open file." << std::endl;
    return false;
  }

  while(*inFile >> thetaBin){
    *inFile >> phiBin >> momBin >> corr_factor;
    proMomCorrs[thetaBin][phiBin][momBin] = corr_factor;
  }

  delete inFile;
  return true;
}

bool ReadInMomCorrs_pip(double pipMomCorrs[12][13][6]) {
  std::ifstream *inFile = 0;
  int thetaBin,phiBin,momBin;
  double corr_factor = 0.0;
  string fileName = "/d/home/skpark/ANALYSIS/MakNtuple/MomCor/mom_corr_array.pip";
  inFile = new std::ifstream(fileName.c_str());

  if(!(inFile->is_open())){
    std::cout << "Error! <ReadInMomCorrs_pip> File read error: " << fileName << " Unable to open file." << std::endl;
    return false;
  }

  while(*inFile >> thetaBin){
    *inFile >> phiBin >> momBin >> corr_factor;
    pipMomCorrs[thetaBin][phiBin][momBin] = corr_factor;
  }

  delete inFile;
  return true;
}

bool ReadInMomCorrs_pim(double pimMomCorrs[12][13][6]) {
  std::ifstream *inFile = 0;
  int thetaBin,phiBin,momBin;
  double corr_factor = 0.0;
  string fileName = "/d/home/skpark/ANALYSIS/MakNtuple/MomCor/mom_corr_array.pim";
  inFile = new std::ifstream(fileName.c_str());

  if(!(inFile->is_open())){
    std::cout << "Error! <ReadInMomCorrs_pim> File read error: " << fileName << " Unable to open file." << std::endl;
    return false;
  }

  while(*inFile >> thetaBin){
    *inFile >> phiBin >> momBin >> corr_factor;
    pimMomCorrs[thetaBin][phiBin][momBin] = corr_factor;
  }

  delete inFile;
  return true;
}

bool ReadInMomCorrs_photon(double photonMomCorrs[12][13][6]) {
  std::ifstream *inFile = 0;
  int thetaBin,phiBin,momBin;
  double corr_factor = 0.0;
  string fileName = "/d/home/skpark/ANALYSIS/MakNtuple/MomCor/mom_corr_array.photon";
  inFile = new std::ifstream(fileName.c_str());

  if(!(inFile->is_open())){
    std::cout << "Error! <ReadInMomCorrs_pim> File read error: " << fileName << " Unable to open file." << std::endl;
    return false;
  }

  while(*inFile >> thetaBin){
    *inFile >> phiBin >> momBin >> corr_factor;
    photonMomCorrs[thetaBin][phiBin][momBin] = corr_factor;
  }

  delete inFile;
  return true;
}
